// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// context.go
package expr

// ---- Functor interface
type Functor interface {
	Invoke(ctx ExprContext, name string, args []any) (result any, err error)
	SetFunc(info ExprFunc)
	GetFunc() ExprFunc
}

// ---- Function Param Info
type ExprFuncParam interface {
	Name() string
	Type() string
	IsOptional() bool
	IsRepeat() bool
	DefaultValue() any
}

// ---- Function Info
type ExprFunc interface {
	Formatter
	Name() string
	MinArgs() int
	MaxArgs() int
	Functor() Functor
	Params() []ExprFuncParam
	ReturnType() string
}

// ----Expression Context
type ExprContext interface {
	Clone() ExprContext
	GetVar(varName string) (value any, exists bool)
	SetVar(varName string, value any)
	UnsafeSetVar(varName string, value any)
	EnumVars(func(name string) (accept bool)) (varNames []string)
	EnumFuncs(func(name string) (accept bool)) (funcNames []string)
	GetFuncInfo(name string) (item ExprFunc, exists bool)
	Call(name string, args []any) (result any, err error)
	// RegisterFunc(name string, f Functor, minArgs, maxArgs int)
	RegisterFuncInfo(info ExprFunc)
	RegisterFunc(name string, f Functor, returnType string, param []ExprFuncParam) error
}
